/*  %kw # %v    %n    %d    %t #             */
/* Version # 4    DMOBAR.C    4-Nov-93    13:55:52 # */
/******************************************************/
/*         Metagraphics Software Corporation           */
/*             Copyright (c) 1987-1993                 */
/*******************************************************/
/* FILE:    DMOBAR.C                                   */
/* PURPOSE: module contains bar graph                  */
/*******************************************************/

#include <stdio.h>

#include "metawndo.h"
#include "GUIconst.h"
#include "GUIextrn.h"

#include "demo.h"

#define Mid(a,b)  (a - b)/2 + b
#define MidPt(r)  Mid(r.Xmax, r.Xmin), Mid(r.Ymax, r.Ymin)

typedef struct  {
   char *label;     /*  Words that go in the bar                   */
   int   size;      /*  Percentage of available space ( 0 - 100 )  */
   int   fColor;    /*  Forground Color                            */
   int   bColor;    /*  Background Color                           */
   int   pattern;   /*  Fill pattern                               */
} BarInfo;

int BACKGROUND;
int LINES     ;
int HEADER    ;
int NUMBERS   ;
int LABELS    ;
int HIGHLIGHT ;

void bargraf()
{
   static char Header[] = " B A R   U S A G E  for  1 9 9 3 ";
   static BarInfo Bars[] =
   {
   { " Foo ",    25,  1,  2, 24 },
   { " Salad ",  80,  4,  5, 23 },
   { " Re ",     48,  7,  8, 30 },
   { " Candy ",  95,  0,  3,  8 },
   { " Crow ",   69,  3,  6, 15 }
   };
   int   NumBars = sizeof (Bars) / sizeof (BarInfo);

   grafPort  *ThisPort;
   rect       tmpR, Bar, BarBox;
   char       tmpstr[64];
   int        i, j, x, y, lnSpacing, numnum, delta, hite, width, size;
   int        Xmin, Xmax, Xmid, Ymin, Ymax;
   
   GetPort ( &ThisPort );
   ProtectRect( &ThisPort->portRect );

   BACKGROUND = 8;
   LINES      = 7;
   HEADER     = 7;
   NUMBERS    = 2;
   LABELS     = 6;
   HIGHLIGHT  = 7;

   if( QueryColors() < 2 ) {
      NUMBERS = 1;
      LABELS  = 1;
   }
   
   PortOrigin ( True );
   Xmin  = ThisPort->portRect.Xmin;   Ymin  = ThisPort->portRect.Ymin;
   Xmax  = ThisPort->portRect.Xmax;   Ymax  = ThisPort->portRect.Ymax;
   Xmid  = (Xmax - Xmin)/2 + Xmin;
   lnSpacing = ThisPort->txFont->lnSpacing;

   /*  EraseScreen  */
   RasterOp( zREPz );
   BackColor( BACKGROUND );
   EraseRect( &ThisPort->portRect );

   /*  Print Headers  */
   TextAlign( alignCenter, alignBottom );
   Shadow( Header, HEADER, Xmid, lnSpacing * 2 + Ymin, 0, 2 );

   /*  Draw right side and bottom  */
   SetRect( &BarBox, StringWidth ("WWWW") * 2 + Xmin,
                     lnSpacing * 3 + Ymin,
                     Xmax - StringWidth ("WWWW"),
                     Ymax - (lnSpacing * 2) );
   PenSize( 3, 2 );
   PenColor( LINES );
   MoveTo( BarBox.Xmin, BarBox.Ymin );
   LineTo( BarBox.Xmin, BarBox.Ymax );
   LineTo( BarBox.Xmax, BarBox.Ymax );

   /*  Print numbers down left side  */
   hite = BarBox.Ymax - BarBox.Ymin + 1;
   numnum = hite / (2 * lnSpacing);
   delta = hite / (numnum - 1);
   x = BarBox.Xmin - 4;
   TextAlign( alignRight, alignMiddle );
   for( y = BarBox.Ymax, i = 0 ; i < numnum ; y -= delta, i++ )  {
      sprintf( tmpstr, "%d", i * 1000 );
      Shadow( tmpstr, NUMBERS, x, y, 0, 1 );
   }

   /*  Draw & label bars  */
   PenSize( penThin );
   width = (BarBox.Xmax - BarBox.Xmin) / (NumBars + 1);
   delta = width / NumBars;
   SetRect( &Bar, BarBox.Xmin + delta, 0,
                  BarBox.Xmin + width + delta, BarBox.Ymax );

   for( i = 0 ; i < NumBars ; i++ )  {
      BackColor( Bars[i].bColor );
      PenColor( Bars[i].fColor );
      PenPattern( Bars[i].pattern);
      if( (size = (hite * Bars[i].size) / 100)  <  lnSpacing * 2 )
         size = lnSpacing * 2;
      Bar.Ymin = BarBox.Ymax - size;

      PaintRect( &Bar );
      PenColor( Black );
      PenPattern( 1 );
      PenColor( 7 );
      MoveTo( Bar.Xmin, Bar.Ymax );
      LineTo( Bar.Xmin, Bar.Ymin );
      LineTo( Bar.Xmax, Bar.Ymin );
      PenColor( Black );
      LineTo( Bar.Xmax, Bar.Ymax );
      LineTo( Bar.Xmin, Bar.Ymax );
      /* FrameRect ( &Bar );*/
      
      /*  put a box around the label  */
      j = StringWidth( Bars[i].label );
      x = Mid( Bar.Xmax, Bar.Xmin );
      y = Mid( Bar.Ymax, Bar.Ymin );
      SetRect ( &tmpR, x - j/2, y - lnSpacing/2, x + j/2, y + lnSpacing/2 );
      BackColor( BACKGROUND );
      EraseRect( &tmpR );
      PenColor( Black );
      MoveTo( tmpR.Xmin, tmpR.Ymax );
      LineTo( tmpR.Xmin, tmpR.Ymin );
      LineTo( tmpR.Xmax, tmpR.Ymin );
      PenColor( 7 );
      LineTo( tmpR.Xmax, tmpR.Ymax );
      LineTo( tmpR.Xmin, tmpR.Ymax );
      /* FrameRect ( &tmpR );*/

      /*  label the bar  */
      /* Shadow( Bars[i].label, LABELS, MidPt (Bar), 0, 1 );*/
      TextAlign( alignCenter, alignMiddle );
      Shadow( Bars[i].label, LABELS, x, y, 0, 1 );

      /*  give the bar a shadow  */
      PenColor( Black );
      PenPattern( 1 );
      SetRect( &tmpR, Bar.Xmax, Bar.Ymin + 5, Bar.Xmax + 4, Bar.Ymax);
      PaintRect( &tmpR );

      /*  setup for next iteration  */
      OffsetRect( &Bar, width + delta, 0 );
   }

   ProtectRect( NULL );


}

void Shadow ( char *Text, int Color, int X, int Y, int Hi, int Sha )
{
   int i;
   
   /*  Draw highlight (upper, left)  */
   if( Hi > 0 ) {
      RasterOp( zREPz );
      PenColor( HIGHLIGHT );
      BackColor( BACKGROUND );
      MoveTo( X-1, Y-1 );
      DrawString( Text );
    }

   /*  Draw Shadow (bottom, right)  */
   RasterOp( zANDz );
   BackColor( White );
   PenColor( Black );

   for( i = Sha ; i >= 0 ; i-- ) {
      MoveTo( X+i, Y+i );
      DrawString( Text );
    }

   /*  Draw the text  */
   MoveTo( X, Y );      TextMode( zORz );
   PenColor( Color );   BackColor( Black );
   DrawString( Text );

   RasterOp( zREPz );

}

