/*  %kw # %v    %n    %d    %t #             */
/* Version # 4    DMOCOLOR.C    10-Nov-93    15:11:24 # */
/******************************************************/
/*         Metagraphics Software Corporation           */
/*             Copyright (c) 1987-1993                 */
/*******************************************************/
/* FILE:    DMOCOLOR.C                                 */
/* PURPOSE: allows palette changes                     */
/*******************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "metawndo.h"
#include "GUIconst.h"
#include "GUIextrn.h"

#include "demo.h"


static long    control[3];
static int     paltype;
static palData palColors[1];
static int     max_colors;
static int     lnSpacing;
static rect    respalbox;


static void draw_palette( void );
static int  check_palette( point * );
static void draw_clr_rect( int );
static void draw_panel( void );
static void draw_control( int );
static void draw_indicator( int );
static int  check_panel( point * );
static void mix_color( int );
static void unmix_color( int );
static int  check_ResetPalette( point * );
static void draw_ResetPaletteBox(void);

void Colorit()
{
   int        selPalEntry;
   int        i, speed;
   rect       scrnR;
   point      cursor_pt;
   grafPort  *gp;
   

   HideCursor();
   BackColor(Black);
   SetRect( &scrnR, 0, 0, 1000, 1000 );
   VirtualRect( &scrnR );
   EraseRect( &scrnR );

   GetPort( &gp );
   cursor_pt.Y = gp->txFont->lnSpacing;
   Lcl2VirPt( &cursor_pt );
   lnSpacing = cursor_pt.Y;

   max_colors = (int) QueryColors();

   paltype = 0;    /* Normal palette ops */

   /* draw the palette */
   draw_palette();

   /* draw the control panel */
   draw_panel();

   /* draw palette reset box */
   draw_ResetPaletteBox();

   ShowCursor();

   /* start with palette entry 0 */
   selPalEntry = 0;
   unmix_color( selPalEntry );
   draw_indicator( 0 );
   draw_indicator( 1 );
   draw_indicator( 2 );
   draw_clr_rect( selPalEntry );


   /* process events */
   while( True ) {
         
      /* wait for an event - but don't get it */
      while( !PeekGUIEvent( 1 ) )
         ;

      /* check close box */
      if( CheckClose( &viewDlgBox ) ) 
         break;

      /* pull it out of the queue */
      KeyGUIEvent( True );

      /* ignore all but mouse press */
      if( !(opEvent.eventType & evntPRESS)  )
         continue;


		cursor_pt.X = opEvent.eventX;
		cursor_pt.Y = opEvent.eventY;

      /* clicked a palette box ? */
      i = check_palette( &cursor_pt );

      if( i >= 0 ) {

         /* select palette entry for editing */
         selPalEntry = i;
         unmix_color( selPalEntry );
         draw_indicator( 0 );
         draw_indicator( 1 );
         draw_indicator( 2 );
         draw_clr_rect( selPalEntry );
         continue;
      }

      /* pressed the reset button ? */
      if( check_ResetPalette( &cursor_pt ) ) {

         /* re-init palette entry for editing */
         unmix_color( selPalEntry );
         draw_indicator( 0 );
         draw_indicator( 1 );
         draw_indicator( 2 );
         draw_clr_rect( selPalEntry );
         continue;
      }

      /* clicked a control ? */
      speed = 1;
      while( (i = check_panel( &cursor_pt )) >= 0  ) {

         /* adjust selected control */
         switch( i ) {
            case 0: control[0]+=speed; i = 0; break;
            case 1: control[0]-=speed; i = 0; break;
            case 2: control[1]+=speed; i = 1; break;
            case 3: control[1]-=speed; i = 1; break;
            case 4: control[2]+=speed; i = 2; break;
            case 5: control[2]-=speed; i = 2; break;
            default: i = 0; break;
         }

         /* accelerate control speed */
         if( speed < 0x7FF )
            speed++;

         /* keep in range */
         if( control[i] > 0xFFFFL )
            control[i] = 0xFFFFL;
         if( control[i] < 0L )
            control[i] = 0L;

         /* update control indicator */
         draw_indicator( i );

         /* update the palette */
         mix_color( selPalEntry );

         /* keep going till they release or move out */
		   KeyGUIEvent( False );
		   if( opEvent.eventButtons == 0 )
            break;
		   cursor_pt.X = opEvent.eventX;
		   cursor_pt.Y = opEvent.eventY;

      } /* end of while on a control */


   }  /* end of while( True ) */

}


void draw_palette()
/*
Draw a screen full of boxes, one for each color combination
*/
{
   int color;
   rect r;
   char buf[10];
   
   MoveTo( 0,0 );

   if( max_colors > 16 ) {

      SetRect( &r, 5, 5, 35, 45 );

      for( color = 0; color <= max_colors; color++ ) {
         
         PenColor( color );
         PaintRect( &r );
         PenColor( White );
         FrameRect( &r );

         OffsetRect( &r, 40, 0 );
         if( r.Xmax > 640 ) {
            r.Xmin = 5; r.Xmax = 35;
            OffsetRect( &r, 0, 50 );
         }
      }

   }
   else {
      
      SetRect( &r, 5, 5, 105, 105 );

      for( color = 0; color <= max_colors; color++ ) {
         
         PenColor( color );
         PaintRect( &r );
         PenColor( White );
         FrameRect( &r );

         MoveTo( r.Xmin + 49, r.Ymax + 35 );
         sprintf( buf, "%d", color );
         DrawString( buf );

         OffsetRect( &r, 150, 0 );
         if( r.Xmax > 599 ) {
            r.Xmin = 5; r.Xmax = 105;
            OffsetRect( &r, 0, 200 );
         }

      }

   }

}

check_palette( point *thepoint )
/*
Check to see if thepoint is in one of the palette display boxes.
Returns the palette entry corresponding to that box, or -1 if not
in any box
*/
{
   int color;
   rect r;
   
   MoveTo( 0,0 );
   if( max_colors > 16 ) {
      SetRect( &r, 5, 5, 35, 45 );
      
      for( color = 0; color <=max_colors; color++ ) {
         
         if( PtInRect( thepoint,  &r, 1,1 ) )
            return color;

         OffsetRect( &r, 40, 0 );
         if( r.Xmax > 640 ) {
            r.Xmin = 5; r.Xmax = 35;
            OffsetRect( &r, 0, 50 );
         }
      }
   }
   else {
      SetRect( &r, 5, 5, 105, 105 );
      for( color = 0; color <=max_colors; color++ ) {
         if( PtInRect( thepoint,  &r, 1,1 ) )
            return color;
         OffsetRect( &r, 150, 0 );
         if( r.Xmax > 599 ) {
            r.Xmin = 5; r.Xmax = 105;
            OffsetRect( &r, 0, 200 );
         }
      }
   }

   return -1;

}

void draw_clr_rect( int palval )
/*
Fill the color mixing rectangle with color 'palval'
*/
{
   rect color_rect;
   char buf[40];
   
   SetRect( &color_rect, 0, 0, 100, 100 );
   OffsetRect( &color_rect, 725, 100 );
   PenColor( palval );
   ProtectRect( &color_rect );
   PaintRect( &color_rect );
   PenColor( White );
   FrameRect( &color_rect );


   sprintf( buf,"%3d: %4X  %4X  %4X", palval,
            palColors[0].palRed, palColors[0].palGreen, palColors[0].palBlue);

   MoveTo( 810, 60 );
   DrawString( buf );
}


void draw_panel()
/*
Draw the control panel
*/
{

   PenColor( White );
   
   draw_control( 0 );
   draw_control( 1 );
   draw_control( 2 );
   draw_clr_rect( 0 );

}

void draw_control( int num )
{
   
   rect r;
   int i, x, y;
   char color;

   switch( num ) {
      
      case 0: x = 670; color = 'R'; break;
      case 1: x = 770; color = 'G'; break;
      case 2: x = 870; color = 'B'; break;
      default: return;
   }

   y = 300;

   /* draw scale */
   PenColor( White );
   MoveTo( x,y );
   LineRel( 0, 500 );

   MoveTo( x,y );
   LineTo( x - 20,y );

   MoveTo( x, y + 500 );
   LineTo( x - 20,y + 500 );

   for( i = y; i < (y + 500); i += 20 ) {
      MoveTo( x,i );
      LineTo( x - 10, i );
   }

   /* draw 'R' 'G' or 'B' */
   MoveTo( x, y - 30 );
   DrawChar( color );


   /* draw '+/-' control */
   y += 520;
   SetRect( &r, x - 30, y , x + 30, y + 80 );
   FrameRect( &r );
   MoveTo( x, y );
   LineTo( x, y + 80 );

   MoveTo( r.Xmin + 15, y + 40 );
   DrawChar( '+' );

   MoveTo( x + 15, y + 40 );
   DrawChar( '-' );

}

void draw_indicator( int num )
/*
Draw a thermometer type indicator bar for control 'num'
*/
{
   int x, y, i;
   rect r;

   x = 670 + 100 * num;
   y = 300;

   /* compute scale factor */
   i = (int) (control[num] / 131L);   /* 0xFFFF / 500 appx = 131 */

   /* fill on part */
   SetRect( &r, x + 1, y + 500 - i, x + 10, y + 500 );
   ProtectRect( &r );
   PaintRect( &r );
   ProtectRect( NULL );

   /* erase off part */
   SetRect( &r, x + 1, y, x + 10, y + 500 - i );
   ProtectRect( &r );
   EraseRect( &r );
   ProtectRect( NULL );
}

int check_panel( point *thepoint )
/*
Check to see if thepoint is in one of the control knobs.
Returns knob number or -1 if not in any knob
*/
{
   int knob, x,y;
   rect r;

   y = 300 + 520;

   for( knob = 0; knob < 3; knob++ ) {

      x = 670 + 100 * knob;

      SetRect( &r, x - 30, y , x + 30, y + 80 );
      if( PtInRect( thepoint, &r, 1,1 ) ) {

         r.Xmin = (r.Xmax + r.Xmin) / 2;
         if( PtInRect( thepoint, &r, 1,1 ) )
            return (knob * 2 + 1);

         return (knob * 2);
       }
   }
   
   return -1;
}


void mix_color( int palval )
/*
For color number entry 'palval' change color register entry
*/

{
   char buf[40];
   
   palColors[0].palRed   = (Word)control[0];
   palColors[0].palGreen = (Word)control[1];
   palColors[0].palBlue  = (Word)control[2];

   sprintf( buf,"%3d: %4X  %4X  %4X", palval,
            palColors[0].palRed, palColors[0].palGreen, palColors[0].palBlue);
   MoveTo( 810, 60 );
   DrawString( buf );
   WritePalette(paltype,palval,palval, &palColors[0]);
}

void unmix_color( int palval )
/*
Get color value of palval and set controls
*/
{
   ReadPalette(paltype,palval,palval, &palColors[0]);
   control[0]= (long)palColors[0].palRed;
   control[1]= (long)palColors[0].palGreen;
   control[2]= (long)palColors[0].palBlue;
}


static void
draw_ResetPaletteBox()
{
   int x,y;
   char *s;

   s = "Reset";
   /* calculate reset button position */
   y = (lnSpacing*3)/2;
   x = StringWidth( s );
   SetRect( &respalbox, 40, 960-y, 60+x, 960 );

   DrawButton( &respalbox, s );
   PenColor( White );
   BackColor( Black );

}


static int check_ResetPalette( point *xy )
{

   if( ! PtInRect( xy, &respalbox, 1,1 ) )
      return False;

   OnButton( &respalbox );
   draw_ResetPaletteBox();

   ResetGUIPalette();

   return True;
}



/* End Of File:  dmocolor.c */
