/*  %kw # %v    %n    %d    %t #             */
/* Version # 6    DMOMAIN.C    5-Aug-94    14:02:02 # */
/*******************************************************/
/*         Metagraphics Software Corporation           */
/*             Copyright (c) 1987-1993                 */
/*******************************************************/
/* FILE:    dmomain.c                                  */
/* PURPOSE: handles init,close,idle and menu functions */
/*          also contains meta menu functions          */
/*******************************************************/

#include <stdio.h>
#include <string.h>

#include "metawndo.h"
#include "GUIconst.h"
#include "GUIextrn.h"

#define DMOMAIN   1
#include "demo.h"

/* Initialize application, called from GUI */
void Init( int argc, char *argv[])
{
   int i;

   /* default to script file demo.scp */
   strcpy( dmoScript, "demo.scp" );

    /* get a file name from command line if present */
   for( i = 1; i < argc; i++ ) {

      /* if its not some kind of option selection */
      if( *(argv[i]) != '/' && *(argv[i]) != '-' ) {

         /* run this name as script */
         strcpy( dmoScript, argv[i] );
         break;
      }

   }

   /* init global script record */
	script.play = script.record = script.forever = False;
	script.dx =  1;
	script.dy =  1;

   /* assume recorded scripts are at VGA res */
   script.sclX =  (long)scrnPort->portRect.Xmax * 100L / 639L;
   script.sclY =  (long)scrnPort->portRect.Ymax * 100L / 480L;
   script.fp = NULL;

   /* startup running automatic script */
   opSelect.selItem = rnAuto;
   RunMenu(); 
   
}



/* Close application, called from GUI */
void Close()
{
   int i;
   
   /* confirm */
   i = Dialog( diaNull, "Ok to quit?", NULL );

   if( i != Ok ) {
	   opSelect.selectID = nullSelect;
      return;
   }

   /* tell GUI to exit */
   opSelect.selectID = exitSelect;

}


/* what to do when waiting for events, called from GUI */
void IdleProcess()
{

   /* process script file */

   if( script.record )
	   ScriptRecord( );

   if( script.play ) 
      ScriptPlay( );

   /* anything else ? */

}


/* Act upon selected menu item, called from GUI */
void ProcessMenu() 
{
   switch( opSelect.selMenu) {

      case mgMenu:
         MetaMenu();
         break;

      case vwMenu:
         ViewMenu();
         break;

      case feMenu:
         FeatureMenu();
         break;

      case spMenu:
         SupportMenu();
         break;

      case ifMenu:
         InfoMenu();
         break;

      case rnMenu:
         RunMenu();
         break;

   }

}

void MetaMenu()
/*
Handle the metagraphics logo menu
*/
{

   dialogBox abDia;
   int x, y, i, line;
   rect bR, lR;
   char mver[80];

   switch( opSelect.selItem ) {
   
      case mgDemo:

         ViewFile( "demo.doc", VERSION );
         break;

      case mgMeta:
         
         // make a dialog box
         line = scrnPort->txFont->lnSpacing;

         /* center it */
         x = scrnPort->portRect.Xmax / 2;
         y = scrnPort->portRect.Ymax / 2;
         i = scrnPort->portRect.Xmax/3;
         SetRect( &bR, x - i, y - (6*line), x+i, y + (6*line ) );


         /* pop up the box */
         DupRect( &bR, &abDia.BoundRect );
         ShowDialog( diaBox, &abDia );

         ProtectRect( &bR );

         /* draw the text strings */
         BackColor( White );
         PenColor( colors->text );
         TextAlign( alignCenter, alignMiddle );
         RasterOp( zREPz );

         y -= 3*line;
         MoveTo(x,y);
         PenColor( colors->bar );
         TextFace( cProportional+cBold );
         DrawString("Metagraphics Software Corporation" );

         PenColor( colors->text );
         TextFace( cProportional );
         y += 2*line; MoveTo(x,y);
         DrawString("P.O. Box 66779");
         y += line; MoveTo(x,y);
         DrawString("Scotts Valley, CA. 95066");
         y += line; MoveTo(x,y);
         DrawString("(800) 332-1550");
         y += line; MoveTo(x,y);
         DrawString("CA (408) 438-1550");
         y += line; MoveTo(x,y);
         DrawString("FAX (408) 438-5379");
         y += line; MoveTo(x,y);
         MetaVersion( mver );
         DrawString( mver );
         ProtectRect( NULL );

         /* wait a bit */
         delay( 70 );

         /* fade out */
         ProtectRect( &bR );
         PenColor( Black );
         RasterOp( zORz );
         for( x = 1; x < 8; x++ ) {
            PenPattern( x );
            PaintRect( &bR );
            delay(2);
         }
         EraseRect( &bR );
         PenPattern( 1 );
         ProtectRect( NULL );

         /* pop dialog off */
         HideDialog( diaBox, &abDia );

         break;

    }


}


void RegenDesk()
/*
Re-generate the desktop
*/
{

   rect tR;

   DupRect( &scrnPort->portRect, &tR );
   tR.Ymin = barRect.Ymax;

   /* clear off desktop */
   PenColor( colors->desktop );
   if( colors->desktop == White )
      BackColor( Black );
   else
      BackColor( White );
   HideCursor();
   FillRect( &tR, colors->deskpat );
   ShowCursor();

   /* redraw any desktop features */

}


/* End of File  -  dmomain.c */
