/*  %kw # %v    %n    %d    %t #             */
/* Version # 5    DMORUN.C    15-Nov-93    16:39:56 # */
/******************************************************/
/*         Metagraphics Software Corporation           */
/*             Copyright (c) 1987-1993                 */
/*******************************************************/
/* FILE:    DMORUN.C                                   */
/* PURPOSE: This module implemrnts the Run menu        */
/*******************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "metawndo.h"
#include "GUIconst.h"
#include "GUIextrn.h"

#include "demo.h"


void RunMenu()
{

   switch( opSelect.selItem ) {
      
      case rnAuto:
         script.forever = True;
         /* fall into run one pass */

      case rnPass:
         
         script.fp = fopen( dmoScript,"rb");
         if( script.fp == NULL ) {
            Dialog( diaAlert, "Can't open script file:", dmoScript );
            script.play = script.forever = False;
            break;
         }
         PeekEvent( 1, &script.evnt );
         script.timebase = opEvent.eventTime;

         /* get first script entry into buffer */
         fread( &script.evnt, sizeof( mwEvent ), 1, script.fp );

         /* set time delta <> real and script */
         script.timebase -= script.evnt.eventTime;

         GUI_menu[ rnMenu ].menuFlags &= ~( 1<<rnRecord );
         script.play = True;

         break;


      case rnRecord:
         
         /* if already recording, turn it off */
         if( script.record ) {
            fclose( script.fp );
            GUI_menu[ rnMenu ].menuFlags |=   1 << rnPass;
            GUI_menu[ rnMenu ].menuFlags |=   1 << rnAuto;
            GUI_menu[ rnMenu ].menuEntry[ rnRecord ] = "&Record";
            script.record = False;
            break;
         }
         
         /* from now on use their script file */
         strcpy( dmoScript, "temp.scp" );

         script.fp = fopen( dmoScript,"wb");
         if( script.fp == NULL ) {
               Dialog( diaAlert, "Can't open script file:", dmoScript );
               break;
         }

         /* do not scale playback from now on */
         script.sclX =  100L;
         script.sclY =  100L;

         PeekEvent( 1, &script.evnt );
         script.lastX = script.evnt.eventX;
         script.lastY = script.evnt.eventY;

         GUI_menu[ rnMenu ].menuFlags &= ~( 1 << rnPass );
         GUI_menu[ rnMenu ].menuFlags &= ~( 1 << rnAuto );
         GUI_menu[ rnMenu ].menuEntry[ rnRecord ] = "&Stop Record";
         script.record = True;

         break;

      case rnQuit:
         Close();   
         break;

    }


}

void ScriptRecord( )
/*
Record to file any event in the queue, or a mouse move
*/
{

   int dx, dy;

   /* if an event ready, record it */
   if( opEvent.eventType != evntNONE )  {
      fwrite( &opEvent, sizeof( mwEvent ), 1, script.fp );
      script.lastX = opEvent.eventX;
      script.lastY = opEvent.eventY;
      return;
   }

   /* no event ready */

   /* if mouse moved enough, record it */
   dx = abs( opEvent.eventX - script.lastX );
   dy = abs( opEvent.eventY - script.lastY );

   if( dx >= script.dx || dy >= script.dy ) {
      opEvent.eventType = evntMOVE;
      fwrite( &opEvent, sizeof( mwEvent ), 1, script.fp );
      script.lastX = opEvent.eventX;
      script.lastY = opEvent.eventY;
   }

}


void ScriptPlay( )
/*
Gets an event from a file and stores it in the queue at the proper time.
If a mouse move event, moves mouse (and doesn't store it)
If a real event has occured (in opEvent), may pause or cancel play.
*/
{

   int x, y;
   static int lastbutton = 0;
   int i, dx, dy;
   unsigned short curtime;
   grafPort *curPort;

   /* real  event ? */
   if( opEvent.eventType != evntNONE && opEvent.eventUser[0] != 0xFF ) {

      if( opEvent.eventChar == ' ' ) {

            /* pause - wait for another space bar */
            do 
               KeyEvent( True, &opEvent );
            while( opEvent.eventChar != ' ' );
    
            return;
      }

      /* return, cancel script play */
      script.play = script.forever = False;
      fclose( script.fp );
      GUI_menu[ rnMenu ].menuFlags |=  1 << rnRecord;
      return;

   } /* end of real event happened */


   /* no real event happened */

   /* keep the button status same as last played event */
   /* in case dragging etc. */
   opEvent.eventButtons = lastbutton;


   /* time for next event to play ? */
   curtime = opEvent.eventTime;
   curtime -= script.timebase;
   if( curtime < (unsigned) script.evnt.eventTime )
      return;

   /* play the event */      

   /* keep returning these buttons until we play the next event */
   lastbutton = script.evnt.eventButtons;

   /* if we are too late to play this event */
   /* recalculate time base from now */
   if( curtime > (unsigned)script.evnt.eventTime + 10 )
      script.timebase = opEvent.eventTime - script.evnt.eventTime;

   /* scale mouse positions for current resolution in use */
   script.evnt.eventX = (short) ( (long) script.evnt.eventX *
                                          script.sclX / 100L);
   script.evnt.eventY = (short) ( (long) script.evnt.eventY *
                                          script.sclY / 100L);

   /* if not a mouse move store it in the queue */
   if( script.evnt.eventType != evntMOVE ) {

      /* mark it as one of our script events */
      script.evnt.eventUser[0] = 0xFF;

      /* convert event coords from whatever to global */
      GetPort( &curPort );
      if( curPort->portFlags & pfVirtual )
         Vir2LclPt( &script.evnt.eventX );
      if( !(curPort->portFlags & pfGblCoord) )
         Lcl2GblPt( &script.evnt.eventX );

      StoreEvent( &script.evnt );
   }
   else {

      /* position mouse */

      /* interpolate positions between current and destination */
      x = opEvent.eventX;
      y = opEvent.eventY;
      dx = script.evnt.eventX - x;
      dy = script.evnt.eventY - y;
      dx /= 4;
      dy /= 4;

      TrackCursor( False );

      if( dx != 0 || dy != 0 ) {
         for( i = 0; i < 3; i++ ) {
            x += dx; y+= dy;
            MoveCursor( x, y );
         }
      }

      MoveCursor( script.evnt.eventX, script.evnt.eventY );

      TrackCursor( True );

   } /* end of handle play event */


   /* update script buffer with next event */
   i = fread( &script.evnt, sizeof( mwEvent ), 1, script.fp );

   /* if done with script, turn play off */
   if( i != 1 ) {

      /* if forever, start up again */
      if( script.forever ) {

         rewind( script.fp );

         PeekEvent( 1, &script.evnt );
         script.timebase = script.evnt.eventTime;

         /* get first script entry into buffer */
         fread( &script.evnt, sizeof( mwEvent ), 1, script.fp );

         /* set time delta <> real and script */
         script.timebase -= script.evnt.eventTime;
      }
      else {
         script.play = False;
         fclose( script.fp );

         /* remove the store of the last event we just did */
         FlushEvents();
         GUI_menu[ rnMenu ].menuFlags |=  1 << rnRecord;
      }

   }  /* end of done with script */


}


