/*  %kw # %v    %n    %d    %t #             */
/* Version # 5    DMOSHOW.C    10-Nov-93    15:17:24 # */
/******************************************************/
/*         Metagraphics Software Corporation           */
/*             Copyright (c) 1987-1993                 */
/******************************************************/
/* FILE:    dmoshow.c                                 */
/* PURPOSE: module contains ball & hilbert            */
/******************************************************/

#include <stdio.h>

#include "metawndo.h"
#include "GUIconst.h"
#include "GUIextrn.h"

#include "demo.h"

void  inc_pen(int);
void  Plt(void);
void  a(int), b(int), c(int), d(int);


/*
** ball expects the event queue to be running for a random #
** Hits is number of wall strikes before next loop.
*/

void ball( int hits, int numloops )
{
   rect     portR,R1,c1;
   grafPort *thePort;
   int      cnt, loop, max_color, xa, ya,i,j;

   GetPort( &thePort );
   DupRect( &thePort->portRect,&portR );
   max_color = (int) QueryColors();

/* calculate the erasing oval size & position */
   c1.Xmin = (portR.Xmax-portR.Xmin)/2;
   c1.Ymin = (portR.Ymax-portR.Ymin)/2;
   c1.Xmax = c1.Xmin+20;
   c1.Ymax = c1.Ymin+20;
   OffsetRect( &c1, -10, -10 );

   RasterOp( zREPz );

   loop = 1;
   while( loop++ ) {

      /* if asking for a specific number of loops ( > 0 ) check it */
      if( loop > numloops && numloops > 0 )
         break;
      
      /* every so often,  erase screen with increasing oval */
      if( loop %4 == 0 ) {
         DupRect(&c1,&R1);
         ProtectRect( &portR );
         while( R1.Xmax < scrnPort->portRect.Xmax ) {
            if( max_color == 1 )
               EraseOval( &R1 );
            else
               PaintOval( &R1 );
            InsetRect( &R1, -(portR.Xmax/20), -(portR.Ymax/20) );
         }
         ProtectRect( NULL );
      }

      /* randomize size & starting position */

      /* i = rand between 8 and 23 */
      PeekGUIEvent( 1 );
      i = (opEvent.eventTime & 0x000F) + 8;


      if( i % 2 ) {   /* set the delta factors */
         xa = i/2; ya = i/3;
      }
      else {
         xa = i/3; ya = i/2;
      }

      j = (opEvent.eventTime & 0x00FF);
      SetRect( &R1, j, j, j+i, j+i );

      cnt = hits;    /* how many bounces to do per loop */
      while( cnt > 0 ) {

         /* anybody press something ? */
         if( PeekGUIEvent( 1 ) ) {
            loop = 0;
            break;
         }

         inc_pen( max_color );
         PaintOval(&R1);

         /* hit the edge ? */
         if( (R1.Xmax >= portR.Xmax) || (R1.Xmin <= portR.Xmin) ) {
            /* flip x delta */
            xa = ~xa;
            cnt--;
            OffsetRect(&R1,xa/2,0);
         }

         /* hit the edge ? */
         if( (R1.Ymax >= portR.Ymax) || (R1.Ymin <= portR.Ymin) ) {
            /* flip y delta */
            ya = ~ya;
            cnt--;
            OffsetRect(&R1,0,ya/2);
         }

         OffsetRect(&R1,xa,ya);

      }  /* end of while cnt */

   }  /* end of while loop */

}

void inc_pen( int max_color)
{
   static int penColr;

   if( max_color > 1 ) {
      penColr++;
      if( penColr > max_color )
         penColr = 1;
      PenColor( penColr );
   }

}


/* Hilbert thing */

static int  x,y,ox,oy,h,hmax_color;

void hilbert()
{

   int         xa,ya,i,loop;
   rect        vR;
   grafPort    *thePort;

   GetPort(&thePort);

   SetRect( &vR, 0,0,256,256 );
   VirtualRect( &vR );
   BackColor( Black );

   hmax_color = (int) QueryColors();

   if( hmax_color > 1 )
      RasterOp(zREPz);
   else {
      RasterOp( zXORz );
      PenColor( White );
   }

   loop = True;

   while( loop ) {

      EraseRect( &vR );

      i = 0; h = 256; ya = xa = h/2;
      while( h > 2) {
         i++;
         h = h/2;
         xa = xa + (h/2);
         ya = ya + (h/2);
         x  = xa;  y = ya;
         ox = xa;  oy = ya;
         a( i );
         if( PeekGUIEvent( 1 ) ) {
            loop = False;
            break;
         }
      }
      inc_pen( hmax_color );
   }

   ProtectRect( &vR );
   for( i = 0; i <= (vR.Ymax - vR.Ymin); i += 16 )
       ScrollRect( &vR, 0, 16 );
   ProtectRect( NULL );
}

void Plt()
{
   MoveTo(ox,oy); LineTo(x,y);
   inc_pen(hmax_color);
   ox=x;  oy=y;
}

void a( int i)
{
   if( i > 0 )  {
      d(i-1);  x=x-h;  Plt();
      a(i-1);  y=y-h;  Plt();
      a(i-1);  x=x+h;  Plt();
      b(i-1);
   }
}

void b( int i)
{
   if( i > 0 ) {
      c(i-1);  y=y+h;  Plt();
      b(i-1);  x=x+h;  Plt();
      b(i-1);  y=y-h;  Plt();
      a(i-1);
   }
}

void c( int i)
{
   if( i > 0 ) {
      b(i-1);  x=x+h;  Plt();
      c(i-1);  y=y+h;  Plt();
      c(i-1);  x=x-h;  Plt();
      d(i-1);
   }
}

void d( int i)
{
   if( i > 0 ) {
      a(i-1);  y=y-h;  Plt();
      d(i-1);  x=x-h;  Plt();
      d(i-1);  y=y+h;  Plt();
      c(i-1);
   }
}

